from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.audit_event_entry import AuditEventEntry


T = TypeVar("T", bound="AuditEventsResponse")


@_attrs_define
class AuditEventsResponse:
    """Response for audit events list.

    Attributes:
        events (list[AuditEventEntry]): Audit events
        total (int): Total events matching filter
        limit (int): Page size
        offset (int): Page offset
    """

    events: list[AuditEventEntry]
    total: int
    limit: int
    offset: int
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        events = []
        for events_item_data in self.events:
            events_item = events_item_data.to_dict()
            events.append(events_item)

        total = self.total

        limit = self.limit

        offset = self.offset

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "events": events,
                "total": total,
                "limit": limit,
                "offset": offset,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.audit_event_entry import AuditEventEntry

        d = dict(src_dict)
        events = []
        _events = d.pop("events")
        for events_item_data in _events:
            events_item = AuditEventEntry.from_dict(events_item_data)

            events.append(events_item)

        total = d.pop("total")

        limit = d.pop("limit")

        offset = d.pop("offset")

        audit_events_response = cls(
            events=events,
            total=total,
            limit=limit,
            offset=offset,
        )

        audit_events_response.additional_properties = d
        return audit_events_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
